﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Devices.Sensors;
using System.Windows.Threading;
using Microsoft.Xna.Framework;
using Microsoft.Devices;

namespace PhoneApp1
{
    public partial class MainPage : PhoneApplicationPage
    {

        Compass compass;
        Accelerometer accelerometer;
        Gyroscope gyroscope;

        DispatcherTimer timer;

        Vector3 vector3 = new Vector3(0, 1, 0);
        //设备对地理北极的偏角
        double trueHeading = 0d;

        public MainPage()
        {
            InitializeComponent();

            Application.Current.Host.Settings.EnableFrameRateCounter = false;

            if (!Compass.IsSupported)
            {
                MessageBox.Show("设备不支持罗盘");
                return;
            }

            if (!Accelerometer.IsSupported)
            {
                MessageBox.Show("设备不支持重力感应");
            }

            if (!Gyroscope.IsSupported)
            {
                MessageBox.Show("设备不支持陀螺仪");
            }


            timer = new DispatcherTimer();
            timer.Interval = TimeSpan.FromMilliseconds(30);
            timer.Tick += new EventHandler(timer_Tick);

        }

        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {
            compass = new Compass();
            //指定数据更新时间间隔.
            compass.TimeBetweenUpdates = TimeSpan.FromMilliseconds(20);
            //从罗盘传感器获取数据时发生
            compass.CurrentValueChanged += new EventHandler<SensorReadingEventArgs<CompassReading>>(compass_CurrentValueChanged);
            //罗盘需要校验时发生
            compass.Calibrate += new EventHandler<CalibrationEventArgs>(compass_Calibrate);


            accelerometer = new Accelerometer();
            accelerometer.TimeBetweenUpdates = TimeSpan.FromMilliseconds(20);
            //从加速度传感器获取数据时发生
            accelerometer.CurrentValueChanged += new EventHandler<SensorReadingEventArgs<AccelerometerReading>>(accelerometer_CurrentValueChanged);


            gyroscope = new Gyroscope();
            gyroscope.TimeBetweenUpdates = TimeSpan.FromMilliseconds(20);
            //从陀螺仪获取数据时发生
            gyroscope.CurrentValueChanged += new EventHandler<SensorReadingEventArgs<GyroscopeReading>>(gyroscope_CurrentValueChanged);


            try
            {
                compass.Start();
                accelerometer.Start();
                timer.Start();
            }
            catch
            {
                msg1.Text = "需要启动罗盘";
            }


        }

        void timer_Tick(object sender, EventArgs e)
        {
            var r = new RotateTransform();
            if (vector3.Z <= 0) r.Angle = -trueHeading;
            else r.Angle = trueHeading;
            image1.RenderTransform = r;

            TX.Text = vector3.X.ToString("0.00");
            TY.Text = vector3.Y.ToString("0.00");
            TZ.Text = vector3.Z.ToString("0.00");
        }
        //从传感器获取新数据时发生
        void compass_CurrentValueChanged(object sender, SensorReadingEventArgs<CompassReading> e)
        {
            //获取到地球地理北极顺时值偏移角度
            trueHeading = e.SensorReading.TrueHeading;
        }

        //罗盘需要校验时发生
        void compass_Calibrate(object sender, CalibrationEventArgs e)
        {
            Dispatcher.BeginInvoke(() =>
            {
                //罗盘需要校验，canvas1指示用户校验
                canvas1.Visibility = Visibility.Visible;
            });
        }

        //完成校验
        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            canvas1.Visibility = Visibility.Collapsed;
        }

        //从加速度传感器获取数据时发生
        void accelerometer_CurrentValueChanged(object sender, SensorReadingEventArgs<AccelerometerReading> e)
        {
            //获取到设备加速度矢量
            vector3 = e.SensorReading.Acceleration;
        }

        //从陀螺仪获取数据是发生
        void gyroscope_CurrentValueChanged(object sender, SensorReadingEventArgs<GyroscopeReading> e)
        {
            //获取设备绕每个轴的旋转速度
            var rotationRate = e.SensorReading.RotationRate;
        }
    }
}